-- lua/gemini/init.lua

local api = require("gemini.api")
local chat = require("gemini.chat")
local config = require("gemini.config")
local completion = require("gemini.completion")

local M = {}

local function get_current_buffer_content()
    local lines = vim.api.nvim_buf_get_lines(0, 0, -1, false)
    return table.concat(lines, "\n")
end

function M.prompt_query(context)
    local prompt = context and "Gemini (with buffer context): " or "Gemini: "
    vim.ui.input({ prompt = prompt }, function(input)
        if input then
            M.query(input, context)
        end
    end)
end

function M.query(prompt, context)
    chat.set_context(context)
    chat.create_window()
    
    local initial_content = "User: " .. prompt .. "\n\nAssistant: Thinking..."
    chat.update_content(initial_content, true, true)
    
    api.get_response(prompt, context, function(response, error)
        if error then
            vim.notify("Failed to get response: " .. error, vim.log.levels.ERROR)
            return
        end

        local content = "User: " .. prompt .. "\n\nAssistant: " .. response
        chat.update_content(content, true)
    end)
end

function M.setup(opts)
    config.setup(opts)
    
    -- Set up completion
    completion.setup()
    
    -- Enable completion by default for all buffers
    vim.api.nvim_create_autocmd("BufEnter", {
        pattern = "*",
        callback = function()
            -- Only enable for specific filetypes
            local ft = vim.bo.filetype
            if not vim.tbl_contains(config.options.completion.exclude_filetypes, ft) then
                vim.b.gemini_completion_enabled = true
            end
        end
    })
    
    -- Add completion trigger keymap
    vim.keymap.set('i', '<C-x><C-g>', function()
        completion.trigger_completion()
    end, { desc = 'Trigger Gemini completion' })
end

function M.complete(findstart, base)
    if findstart == 1 then
        local line = vim.api.nvim_get_current_line()
        local col = vim.api.nvim_win_get_cursor(0)[2]
        local start = col
        while start > 0 and string.match(line:sub(start, start), "[%w_]") do
            start = start - 1
        end
        return start
    else
        -- Don't return "Loading..." immediately
        -- Instead, return empty list and let completion handle it
        return {}
    end
end

return M
